@@ -2,6 +2,8 @@
 
 require 'thread'
 require 'digest/sha1'
+require 'timeout'
+require 'forwardable'
 
 module Stomp
 
@@ -11,27 +13,14 @@ module Stomp
   # Receives all happen in one thread, so consider not doing much processing
   # in that thread if you have much message volume.
   class Client
+    extend Forwardable
 
-    public
-
-    # The login ID used by the client.
-    attr_reader :login
-
-    # The login credentials used by the client.
-    attr_reader :passcode
-
-    # The Stomp host specified by the client.
-    attr_reader :host
-
-    # The Stomp host's listening port.
-    attr_reader :port
-
-    # Is this connection reliable?
-    attr_reader :reliable
-
-    # Parameters Hash, possibly nil for a non-hashed connect.
+    # Parameters hash
     attr_reader :parameters
 
+    def_delegators :@connection, :login, :passcode, :port, :host, :ssl
+    def_delegator :@parameters, :reliable
+
     # A new Client object can be initialized using three forms:
     #
     # Hash (this is the recommended Client initialization method):
@@ -57,6 +46,10 @@ module Stomp
     #     :hbser => false,
     #     :stompconn => false,
     #     :usecrlf => false,
+    #     :max_hbread_fails => 0,
+    #     :max_hbrlck_fails => 0,
+    #     :fast_hbs_adjust => 0.0,
+    #     :connread_timeout => 0,
     #   }
     #
     #   e.g. c = Stomp::Client.new(hash)
@@ -81,70 +74,46 @@ module Stomp
     #   e.g. c = Stomp::Client.new(urlstring)
     #
     def initialize(login = '', passcode = '', host = 'localhost', port = 61613, reliable = false, autoflush = false)
-
-      # Parse stomp:// URL's or set params
-      if login.is_a?(Hash)
-        @parameters = login
-
-        first_host = @parameters[:hosts][0]
-
-        @login = first_host[:login]
-        @passcode = first_host[:passcode]
-        @host = first_host[:host]
-        @port = first_host[:port] || Connection::default_port(first_host[:ssl])
-
-        @reliable = true
-      elsif login =~ /^stomp:\/\/#{url_regex}/ # e.g. stomp://login:passcode@host:port or stomp://host:port
-        @login = $2 || ""
-        @passcode = $3 || ""
-        @host = $4
-        @port = $5.to_i
-        @reliable = false
-      elsif login =~ /^failover:(\/\/)?\(stomp(\+ssl)?:\/\/#{url_regex}(,stomp(\+ssl)?:\/\/#{url_regex}\))+(\?(.*))?$/ 
-        # e.g. failover://(stomp://login1:passcode1@localhost:61616,stomp://login2:passcode2@remotehost:61617)?option1=param
-        first_host = {}
-        first_host[:ssl] = !$2.nil?
-        @login = first_host[:login] = $4 || ""
-        @passcode = first_host[:passcode] = $5 || ""
-        @host = first_host[:host] = $6
-        @port = first_host[:port] = $7.to_i || Connection::default_port(first_host[:ssl])
-
-        options = $16 || ""
-        parts = options.split(/&|=/)
-        options = Hash[*parts]
-
-        hosts = [first_host] + parse_hosts(login)
-
-        @parameters = {}
-        @parameters[:hosts] = hosts
-
-        @parameters.merge! filter_options(options)
-
-        @reliable = true
-      else
-        @login = login
-        @passcode = passcode
-        @host = host
-        @port = port.to_i
-        @reliable = reliable
-      end
+      parse_hash_params(login) ||
+        parse_stomp_url(login) ||
+        parse_failover_url(login) ||
+        parse_positional_params(login, passcode, host, port, reliable)
 
       check_arguments!()
 
-      @id_mutex = Mutex.new()
-      @ids = 1
+      @logger = @parameters[:logger] ||= Stomp::NullLogger.new
 
-      if @parameters
-        @connection = Connection.new(@parameters)
-      else
-        @connection = Connection.new(@login, @passcode, @host, @port, @reliable)
-        @connection.autoflush = autoflush
+      @start_timeout = @parameters[:start_timeout] || 10
+      Timeout.timeout(@start_timeout, Stomp::Error::StartTimeoutException.new(@start_timeout)) do
+        create_error_handler
+        create_connection(autoflush)
+        start_listeners()
       end
-
-      start_listeners()
-
     end
 
+    def create_error_handler
+      client_thread = Thread.current
+
+      @error_listener = lambda do |error|
+        exception = case error.body
+                      when /ResourceAllocationException/i
+                        Stomp::Error::ProducerFlowControlException.new(error)
+                      when /ProtocolException/i
+                        Stomp::Error::ProtocolException.new(error)
+                      else
+                        Stomp::Error::BrokerException.new(error)
+                    end
+
+        client_thread.raise exception
+      end
+    end
+
+    def create_connection(autoflush)
+      @connection = Connection.new(@parameters)
+      @connection.autoflush = autoflush
+    end
+    private :create_connection
+
     # open is syntactic sugar for 'Client.new', see 'initialize' for usage.
     def self.open(login = '', passcode = '', host = 'localhost', port = 61613, reliable = false)
       Client.new(login, passcode, host, port, reliable)
@@ -169,9 +138,7 @@ module Stomp
       replay_list = @replay_messages_by_txn[name]
       if replay_list
         replay_list.each do |message|
-          if listener = find_listener(message)
-            listener.call(message)
-          end
+          find_listener(message) # find_listener also calls the listener
         end
       end
     end
@@ -209,7 +176,7 @@ module Stomp
     # Acknowledge a message, used when a subscription has specified
     # client acknowledgement ( connection.subscribe("/queue/a",{:ack => 'client'}).
     # Accepts a transaction header ( :transaction => 'some_transaction_id' ).
-    def acknowledge(message, headers = {})
+    def ack(message, headers = {})
       txn_id = headers[:transaction]
       if txn_id
         # lets keep around messages ack'd in this transaction in case we rollback
@@ -225,14 +192,20 @@ module Stomp
       end
       if protocol() == Stomp::SPL_12
         @connection.ack(message.headers['ack'], headers)
+      elsif protocol == Stomp::SPL_11
+        headers.merge!(:subscription => message.headers['subscription'])
+        @connection.ack(message.headers['message-id'], headers)
       else
         @connection.ack(message.headers['message-id'], headers)
       end
     end
 
+    # For posterity, we alias:
+    alias acknowledge ack
+
     # Stomp 1.1+ NACK.
-    def nack(message_id, headers = {})
-      @connection.nack(message_id, headers)
+    def nack(message, headers = {})
+      @connection.nack(message, headers)
     end
 
     # Unreceive a message, sending it back to its queue or to the DLQ.
@@ -271,6 +244,11 @@ module Stomp
       @connection.closed?()
     end
 
+    # jruby? tests if the connection has detcted a JRuby environment
+    def jruby?()
+      @connection.jruby
+    end
+
     # close frees resources in use by this client.  The listener thread is
     # terminated, and disconnect on the connection is called.
     def close(headers={})
@@ -285,6 +263,7 @@ module Stomp
 
     # set_logger identifies a new callback logger.
     def set_logger(logger)
+      @logger = logger
       @connection.set_logger(logger)
     end
 
