@@ -17,16 +17,24 @@ describe Stomp::Connection do
       :back_off_multiplier => 2,
       :max_reconnect_attempts => 0,
       :randomize => false,
-      :backup => false,
       :connect_timeout => 0,
       :parse_timeout => 5,
       :connect_headers => {},
-      :dmh => false
-    }
+      :dmh => false,
+      :closed_check => true,
+      :hbser => false,
+      :stompconn => false,
+      :usecrlf => false,
+      :max_hbread_fails => 0,
+      :max_hbrlck_fails => 0,
+      :fast_hbs_adjust => 0.0,
+      :connread_timeout => 0,
+      :tcp_nodelay => true,
+   }
         
     #POG:
     class Stomp::Connection
-      def _receive( s )
+      def _receive( s, connread = false )
       end
     end
     
@@ -52,7 +60,6 @@ describe Stomp::Connection do
         "backOffMultiplier" => 2,
         "maxReconnectAttempts" => 0,
         "randomize" => false,
-        "backup" => false,
         "connect_timeout" => 0,
         "parse_timeout" => 5,
       }
@@ -70,7 +77,7 @@ describe Stomp::Connection do
       @connection.publish "/queue", "message", :suppress_content_length => false
     end    
   end
-  
+
   describe "(created using a hash)" do
     it "should uncamelize and symbolize the main hash keys" do
       used_hash = {
@@ -85,9 +92,13 @@ describe Stomp::Connection do
         "backOffMultiplier" => 2,
         "maxReconnectAttempts" => 0,
         "randomize" => false,
-        "backup" => false,
-        "connect_timeout" => 0,
-        "parse_timeout" => 5
+        "connectTimeout" => 0,
+        "parseTimeout" => 5,
+        "usecrlf" => false,
+        :maxHbreadFails => 0,
+        :maxHbrlckFails => 0,
+        :fastHbsAdjust => 0.0,
+        :connreadTimeout => 0,
       }
       
       @connection = Stomp::Connection.new(used_hash)
@@ -99,7 +110,7 @@ describe Stomp::Connection do
     end
     
     it "should change host to next one with randomize false" do
-      @connection.change_host
+      @connection.send(:change_host) # use .send(:name) to test a private method!
       @connection.instance_variable_get(:@host).should == "remotehost"
     end
     
@@ -149,7 +160,7 @@ describe Stomp::Connection do
         @connection.publish "/queue", "сообщение"  # 'сообщение' is 'message' in Russian
       end
     end
-    
+
     describe "when unacknowledging a message" do
       
       before :each do
@@ -209,22 +220,23 @@ describe Stomp::Connection do
         @message.headers[:retry_count].should == 5
       end
       
-      it "should not send the message to the dead letter queue as persistent if redeliveries equal max redeliveries" do
+      it "should not send the message to the dead letter queue as persistent if retry_count is less than max redeliveries" do
         max_redeliveries = 5
         dead_letter_queue = "/queue/Dead"
         
-        @message.headers["retry_count"] = max_redeliveries
+        @message.headers["retry_count"] = max_redeliveries - 1
         transaction_id = "transaction-#{@message.headers["message-id"]}-#{@message.headers["retry_count"]}"
         @retry_headers = @retry_headers.merge :transaction => transaction_id, :retry_count => @message.headers["retry_count"] + 1
         @connection.should_receive(:publish).with(@message.headers["destination"], @message.body, @retry_headers)
         @connection.unreceive @message, :dead_letter_queue => dead_letter_queue, :max_redeliveries => max_redeliveries
       end
       
+      # If the retry_count has reached max_redeliveries, then we're done.
       it "should send the message to the dead letter queue as persistent if max redeliveries have been reached" do
         max_redeliveries = 5
         dead_letter_queue = "/queue/Dead"
         
-        @message.headers["retry_count"] = max_redeliveries + 1
+        @message.headers["retry_count"] = max_redeliveries
         transaction_id = "transaction-#{@message.headers["message-id"]}-#{@message.headers["retry_count"]}"
         @retry_headers = @retry_headers.merge :persistent => true, :transaction => transaction_id, :retry_count => @message.headers["retry_count"] + 1, :original_destination=> @message.headers["destination"]
         @connection.should_receive(:publish).with(dead_letter_queue, @message.body, @retry_headers)
@@ -238,7 +250,7 @@ describe Stomp::Connection do
       end
     
     end
-    
+
     describe "when sending a nil message body" do
       it "should should not raise an error" do
         @connection = Stomp::Connection.new("niluser", "nilpass", "localhost", 61613)
@@ -266,7 +278,9 @@ describe Stomp::Connection do
       
       before(:each) do
         ssl_parameters = {:hosts => [{:login => "login2", :passcode => "passcode2", :host => "remotehost", :ssl => true}]}
-        @ssl_socket = mock(:ssl_socket, :puts => nil, :write => nil, :setsockopt => nil, :flush => true)
+        @ssl_socket = mock(:ssl_socket, :puts => nil, :write => nil, 
+          :setsockopt => nil, :flush => true)
+        @ssl_socket.stub!(:sync_close=)
         
         TCPSocket.should_receive(:open).and_return @tcp_socket
         OpenSSL::SSL::SSLSocket.should_receive(:new).and_return(@ssl_socket)
@@ -287,21 +301,21 @@ describe Stomp::Connection do
 
     describe "when called to increase reconnect delay" do
       it "should exponentialy increase when use_exponential_back_off is true" do
-        @connection.increase_reconnect_delay.should == 0.02
-        @connection.increase_reconnect_delay.should == 0.04
-        @connection.increase_reconnect_delay.should == 0.08
+        @connection.send(:increase_reconnect_delay).should == 0.02
+        @connection.send(:increase_reconnect_delay).should == 0.04
+        @connection.send(:increase_reconnect_delay).should == 0.08
       end
       it "should not increase when use_exponential_back_off is false" do
         @parameters[:use_exponential_back_off] = false
         @connection = Stomp::Connection.new(@parameters)
-        @connection.increase_reconnect_delay.should == 0.01
-        @connection.increase_reconnect_delay.should == 0.01
+        @connection.send(:increase_reconnect_delay).should == 0.01
+        @connection.send(:increase_reconnect_delay).should == 0.01
       end
       it "should not increase when max_reconnect_delay is reached" do
         @parameters[:initial_reconnect_delay] = 8.0
         @connection = Stomp::Connection.new(@parameters)
-        @connection.increase_reconnect_delay.should == 16.0
-        @connection.increase_reconnect_delay.should == 30.0
+        @connection.send(:increase_reconnect_delay).should == 16.0
+        @connection.send(:increase_reconnect_delay).should == 30.0
       end
       
       it "should change to next host on socket error" do
@@ -311,7 +325,7 @@ describe Stomp::Connection do
         #tries the new host
         TCPSocket.should_receive(:open).and_return @tcp_socket
 
-        @connection.socket
+        @connection.send(:socket)
         @connection.instance_variable_get(:@host).should == "remotehost"
       end
       
@@ -329,11 +343,18 @@ describe Stomp::Connection do
           :back_off_multiplier => 2,
           :max_reconnect_attempts => 0,
           :randomize => false,
-          :backup => false,
           :connect_timeout => 0,
           :parse_timeout => 5,
           :connect_headers => {},
           :dmh => false,
+          :closed_check => true,
+          :hbser => false,
+          :stompconn => false,
+          :max_hbread_fails => 0,
+          :max_hbrlck_fails => 0,
+          :fast_hbs_adjust => 0.0,
+          :connread_timeout => 0,
+          :tcp_nodelay => true,
         }
         
         used_hash =  {
@@ -359,7 +380,6 @@ describe Stomp::Connection do
           :back_off_multiplier => 3,
           :max_reconnect_attempts => 10,
           :randomize => true,
-          :backup => false,
           :reliable => false,
           :connect_timeout => 0,
           :parse_timeout => 20,
@@ -367,12 +387,21 @@ describe Stomp::Connection do
           :dead_letter_queue => "queue/Error",
           :max_redeliveries => 10,
           :dmh => false,
+          :closed_check => true,
+          :hbser => true,
+          :stompconn => true,
+          :usecrlf => true,
+          :max_hbread_fails => 123,
+          :max_hbrlck_fails => 456,
+          :fast_hbs_adjust => 0.2,
+          :connread_timeout => 42,
+          :tcp_nodelay => true,
         }
         
         @connection = Stomp::Connection.new(used_hash)
         received_hash = @connection.instance_variable_get(:@parameters)
         
-        #Using randomize we can't assure the hosts order
+        # Using randomize we can't assure the hosts order
         received_hash.delete(:hosts)
         used_hash.delete(:hosts)
         
@@ -386,11 +415,11 @@ describe Stomp::Connection do
   describe "when closing a socket" do
     it "should close the tcp connection" do
       @tcp_socket.should_receive(:close)
-      @connection.obj_send(:close_socket).should be_true
+      @connection.__send__(:close_socket).should be_true # Use Object.__send__
     end
     it "should ignore exceptions" do
       @tcp_socket.should_receive(:close).and_raise "exception"
-      @connection.obj_send(:close_socket).should be_true
+      @connection.__send__(:close_socket).should be_true # Use Object.__send__
     end
   end
 
@@ -399,11 +428,11 @@ describe Stomp::Connection do
       host = @parameters[:hosts][0]
       @connection = Stomp::Connection.new(host[:login], host[:passcode], host[:host], host[:port], reliable = true, 5, connect_headers = {})
       @connection.instance_variable_set(:@connection_attempts, 10000)
-      @connection.max_reconnect_attempts?.should be_false
+      @connection.send(:max_reconnect_attempts?).should be_false
     end
     it "should return false if max_reconnect_attempts = 0" do
       @connection.instance_variable_set(:@connection_attempts, 10000)
-      @connection.max_reconnect_attempts?.should be_false
+      @connection.send(:max_reconnect_attempts?).should be_false
     end
     it "should return true if connection attempts > max_reconnect_attempts" do
       limit = 10000
@@ -411,12 +440,19 @@ describe Stomp::Connection do
       @connection = Stomp::Connection.new(@parameters)
       
       @connection.instance_variable_set(:@connection_attempts, limit-1)
-      @connection.max_reconnect_attempts?.should be_false
+      @connection.send(:max_reconnect_attempts?).should be_false
       
       @connection.instance_variable_set(:@connection_attempts, limit)
-      @connection.max_reconnect_attempts?.should be_true
+      @connection.send(:max_reconnect_attempts?).should be_true
+    end
+    # These should be raised for the user to deal with
+    it "should not rescue MaxReconnectAttempts" do
+      @connection = Stomp::Connection.new(@parameters)
+      @connection.stub(:socket).and_raise(Stomp::Error::MaxReconnectAttempts)
       
+      expect { @connection.receive() }.to raise_error
     end
   end
+
 end
 
